local _,stash = ...
local L = stash.localization
local C = stash.constants
local settings = stash.settings
stash.settingsPanel = stash.frame.settingsPanel

stash.settingsPanel.funcs = {} -- indexed by var, functions to run when var changes

-- this is the list used for the scrollframe. a list of characters to reset is appended to it and
-- then version after that:  {type, key, text, hasSettingsFunc}
--    [1] = type: "check", "spacer", "header", "reset" or "version"
--    [2] = key: the settings.var key like settings.LockWindow
--    [3] = text: the name of the setting
--    [4] = hasFunc: whether there's a stash.settingsPanel.funcs[key] to run when the option changes/on login
--    [5] = realm: for character reset, the realm of the character to reset
--    [6] = character: for character reset, the name of the character to reset
--    [7] = lastSeen: for character reset, when the character last logged in
stash.settingsPanel.settingsInfo = {
    {"check","LockWindow",L["Lock window size and position"],true},
    {"check","PerCharacterPosition",L["Remember size and position per character"],true},
    {"check","NoESCape",L["Don't dismiss window with ESC key"],false},
    {"check","ResetOnClose",L["Reset filters when window dimissed"],false},
    {"check","ShowMinimapButton",L["Use minimap button to summon window"],true},
    {"check","ShowRarityBorders",L["Show item rarity borders"],false},
    {"check","HideMoneyTooltip",L["Hide tooltip for gold per character"],false},
    {"check","DefaultSearchTypes",L["Default to search types too"],true},
    {"check","DefaultCurrentRealm",L["Default to current realm & faction"],true},
    {"check","AutoFocus",L["Automatically grab searchbox focus"],false},
    {"check","TabFocus",L["Allow Tab key to grab searchbox focus"],false},
    {"check","DontConfirmReset",L["Don't ask to confirm character resets"],false},
    {"check","AddToTooltips",L["Add stash details to tooltips everywhere"],true},
    {"header",nil,L["Reset a character's stash"]},
}

-- when the settings list is populated everything up to numSettingsInfo should not be wiped
stash.settingsPanel.numSettingsInfo = #stash.settingsPanel.settingsInfo

function stash.settingsPanel:Setup()
    self.Top.Title:SetText(L["Settings"])
    self.Bottom.BackButton.Text:SetText(OKAY)

	local scrollFrame = self.List
	scrollFrame.template = "StashSettingsListButtonTemplate"
	scrollFrame.callback = self.FillListButton
    scrollFrame.list = self.settingsInfo
end

-- update for settings panel
function stash.settingsPanel:Update()
    self.Bottom:SetPoint("BOTTOMRIGHT",settings.LockWindow and 0 or -24, 0)
    self:PopulateList()
    self.List:Update()
end

function stash.settingsPanel:OnShow()
    self:Update()
end

function stash.settingsPanel:PopulateList()
    local list = self.settingsInfo
    -- remove everything after "Reset a character's stash:" and re-fill it
    for i=#list,self.numSettingsInfo+1,-1 do
        tremove(list,i)
    end
    -- now add all realm/characters to settingsInfo
    for realm,realmInfo in pairs(stash.saved) do
        for player,playerInfo in pairs(realmInfo) do
            tinsert(list,{"reset",nil,nil,nil,realm,player,playerInfo.update})
        end
    end
    -- now do a simple sort of the "reset" entries (not using table.sort since it's an unstable sort for non-reset stuff)
    local unsorted = true
    local count = 0
    while unsorted and count < 10000 do -- in case of runaway sort for some reason (seems impossible but just being safe)
        unsorted = false -- assume everything sorted
        for i=2,#list do
            if list[i][1]=="reset" and list[i-1][1]=="reset" then -- only sorting "reset" info types
                -- if realm same and character2<character1 or if realm2<realm1 then swap
                if (list[i][5]==list[i-1][5] and list[i][6]<list[i-1][6]) or (list[i][5]<list[i-1][5]) then 
                    local realm1,character1,lastSeen1 = list[i-1][5],list[i-1][6],list[i-1][7]
                    local realm2,character2,lastSeen2 = list[i][5],list[i][6],list[i][7]
                    list[i-1][5],list[i-1][6],list[i-1][7] = realm2,character2,lastSeen2
                    list[i][5],list[i][6],list[i][7] = realm1,character1,lastSeen1
                    unsorted = true -- swap made, do another sort pass
                    count = count + 1
                end
            end
        end
    end
    -- one all resets added and sorted, add a version at very end
    tinsert(list,{"version"})
end

function stash.settingsPanel.Top.SettingsToggle:OnClick()
    stash.settingsPanel:Hide()
    stash.itemsPanel:Show()
end

function stash.settingsPanel:FillListButton(info)
    local infoType,var,text,hasFunc,realm,character,lastSeen = info[1],info[2],info[3],info[4],info[5],info[6],info[7]
    if infoType=="spacer" then
        self.LastSeen:Hide()
        self.Text:Hide()
        self.CheckButton:Hide()
        self.Highlight:Hide()
        self.ResetButton:Hide()
    elseif infoType=="check" then
        self.LastSeen:Hide()
        self.Text:SetPoint("LEFT",26,0)
        self.Text:SetPoint("RIGHT",-4,0)
        self.Text:SetTextColor(0.9,0.9,0.9)
        self.Text:SetText(text)
        self.Text:Show()
        self.CheckButton:Show()
        self.CheckButton:SetChecked(settings[var] and true)
        self.CheckButton:SetHitRectInsets(-2,-self:GetWidth()+24,0,0)
        self.ResetButton:Hide()
    elseif infoType=="header" then
        self.LastSeen:SetText(L["Last Seen"])
        self.LastSeen:SetTextColor(1,0.82,0)
        self.LastSeen:Show()
        self.Text:SetPoint("LEFT",4,0)
        self.Text:SetPoint("RIGHT",self.LastSeen,"LEFT",-2,0)
        self.Text:SetTextColor(1,0.82,0)
        self.Text:SetText(text)
        self.Text:Show()
        self.CheckButton:Hide()
        self.ResetButton:Hide()
    elseif infoType=="reset" then
        self.LastSeen:SetText(lastSeen)
        self.LastSeen:SetTextColor(0.9,0.9,0.9)
        self.LastSeen:Show()
        self.Text:SetPoint("LEFT",4,0)
        self.Text:SetPoint("RIGHT",self.LastSeen,"LEFT",-2,0)
        local color = C.FACTION_COLORS[stash.saved[realm][character].faction] or C.FACTION_COLORS.Neutral
        self.Text:SetText(format("%s%s-%s",color,character,realm))
        self.Text:Show()
        self.CheckButton:Hide()
        self.ResetButton:Show()
        self.ResetButton.realm = realm
        self.ResetButton.character = character
    elseif infoType=="version" then
        self.LastSeen:Hide()
        self.Text:SetPoint("LEFT",4,0)
        self.Text:SetPoint("RIGHT",-4,0)
        self.Text:SetTextColor(0.5,0.5,0.5)
        self.Text:SetText(format(L["Stash version %s"],GetAddOnMetadata("Stash","Version")))
        self.Text:Show()
        self.CheckButton:Hide()
        self.ResetButton:Hide()
    end
end

function stash.settingsPanel:ListButtonOnEnter()
    if self.index>stash.settingsPanel.numSettingsInfo then
        self.Highlight:Show()
    end
end

function stash.settingsPanel:ListButtonOnLeave()
    self.Highlight:Hide()
end

function stash.settingsPanel:CheckButtonOnClick()
    local info = stash.settingsPanel.settingsInfo[self:GetParent().index]
    if info[1]=="check" then
        settings[info[2]] = self:GetChecked()
        if info[4] and stash.settingsPanel.funcs[info[2]] then
            stash.settingsPanel.funcs[info[2]](self)
        end
    end
end

function stash.settingsPanel:ResetButtonOnClick()
    stash.confirmPanel.realm = self.realm
    stash.confirmPanel.character = self.character

    -- if DontConfirmReset enabled, then skip confirmation panel
    if settings.DontConfirmReset then
        stash.confirmPanel:AcceptButtonOnClick()
        stash.settingsPanel:Update()
    else -- otherwise show confirmation panel
        stash.settingsPanel:Hide()
        stash.confirmPanel:Show()
    end
end

-- settings funcs to run when a var changes

function stash.settingsPanel.funcs:LockWindow()
    stash.frame:Update()
    stash.settingsPanel:Update()
end

function stash.settingsPanel.funcs:PerCharacterPosition()
    stash.frame:RememberPosition()
end

function stash.settingsPanel.funcs:DefaultSearchTypes()
    settings.SearchTypesToo = self:GetChecked()
end

function stash.settingsPanel.funcs:DefaultCurrentRealm()
    settings.CurrentRealmOnly = self:GetChecked()
end

function stash.settingsPanel.funcs:ShowMinimapButton()
    stash.minimapButton:SetShown(self:GetChecked())
end

function stash.settingsPanel.funcs:AddToTooltips()
    stash.tooltips:UpdateTooltipHooks()
end